 

package com.sonicsw.pso.demo.parsers;

import static com.sonicsw.pso.demo.util.IConstants.*;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.sonicsw.pso.demo.config.Configuration;
import com.sonicsw.pso.demo.util.MessageRootMapConverter;
import com.sonicsw.xq.XQConstants;
import com.sonicsw.xq.XQEnvelope;
import com.sonicsw.xq.XQInitContext;
import com.sonicsw.xq.XQLog;
import com.sonicsw.xq.XQMessage;
import com.sonicsw.xq.XQMessageException;
import com.sonicsw.xq.XQMessageFactory;
import com.sonicsw.xq.XQParameterInfo;
import com.sonicsw.xq.XQParameters;
import com.sonicsw.xq.XQPart;
import com.sonicsw.xq.XQServiceContext;
import com.sonicsw.xq.XQServiceEx;
import com.sonicsw.xq.XQServiceException;
import com.thoughtworks.xstream.XStream;

public class ParseHTMLForm implements XQServiceEx {

    private XQInitContext m_initialContext;
	// This is the XQLog (the container's logging mechanism).
    private XQLog m_xqLog = null;

    //This is the the log prefix that helps identify this service during logging
    private String m_logPrefix = "";
    
    //These hold version information.
    private static int s_major = 1;
    private static int s_minor = 0;
    private static int s_buildNumber = 11;
    
    private Configuration m_configObject = null;
    private XStream m_xstream =null;
    
    private int m_outputMessagePart = 0;
    private int m_inputMessagePart = 0;
    private static final String CONFIG="config"; 
    private static final String OUTPUT_MESSAGE_PART="OutputMessagePart"; 
    private static final String INPUT_MESSAGE_PART="InputMessagePart"; 
    /**
     * Constructor for a ParseHTMLForm
     */
	public ParseHTMLForm () {	
	}
	
	/**
     * Initialize the XQService by processing its initialization parameters.
     *
     * <p> This method implements a required XQService method.
     *
     * @param initialContext The Initial Service Context provides access to:<br>
     * <ul>
     * <li>The configuration parameters for this instance of the ParseHTMLForm.</li>
     * <li>The XQLog for this instance of the ParseHTMLForm.</li>
     * </ul>
     * @exception XQServiceException Used in the event of some error.
     */
    public void init(XQInitContext initialContext) throws XQServiceException {
        m_initialContext = initialContext;
        XQParameters params = m_initialContext.getParameters();
        m_xqLog = m_initialContext.getLog();
        setLogPrefix(params);
        m_xqLog.logInformation(m_logPrefix + " Initializing ...");
        m_xstream = new XStream();
        m_xstream.alias("Configuration", Configuration.class);
        m_xstream.registerConverter(new MessageRootMapConverter(m_xstream.getMapper()));

        
        writeStartupMessage(params);
        writeParameters(params);
        //perform initialization work.
        m_outputMessagePart  = params.getIntParameter(OUTPUT_MESSAGE_PART, 
                XQConstants.PARAM_STRING);
        m_inputMessagePart  = params.getIntParameter(INPUT_MESSAGE_PART, 
                XQConstants.PARAM_STRING);
        
        m_configObject = loadConfigurationObject(params,true);
        if (m_configObject==null) {
            m_configObject = new Configuration();
            m_configObject.add("Key");
            m_configObject.add("Message");
        }
        m_xstream.alias(m_configObject.getRoot(), Map.class);

        m_xqLog.logInformation(m_logPrefix +" Initialized ...");
    }

    /**
     * @param params
     * @return
     */
    private Configuration loadConfigurationObject(XQParameters params,boolean isInit) {
        Configuration conf = null;
        
        XQParameterInfo info = params.getInfo(CONFIG, XQConstants.PARAM_STRING);
        String config  = null;
        if (info!=null) {
            config  = info.getRef();            
        } else {
            if (isInit) {
                m_xqLog.logInformation(m_logPrefix +" got "+CONFIG+" url = NULL"    );
            }
            return null;
        }
        //String config  = info.getRef();
        
        m_xqLog.logInformation(m_logPrefix +" got "+CONFIG+" url ="+config);
        // xStream.createObjectInputStream(new java.io.FileInputStream()));
        try {
            URL url = (new URL(config));
            conf = (Configuration) m_xstream.fromXML(url.openStream());
        } catch (FileNotFoundException ex) {
            m_xqLog.logError("Caught FileNotFoundException Message " + 
            	ex.toString());
            m_xqLog.logError(ex);
            //ex.printStackTrace(System.out);           
        } catch (MalformedURLException ex) {
            m_xqLog.logError("Caught MalformedURLException Message " + 
            	ex.toString());
            m_xqLog.logError(ex);
            //ex.printStackTrace(System.out);
            
        } catch (IOException ex) {
            m_xqLog.logError("Caught IOException Message " + 
            	ex.toString());
            m_xqLog.logError(ex);
            //ex.printStackTrace(System.out);
            
        }
        return conf;
    }


    /**
     * Handle the arrival of XQMessages in the INBOX.
     *
     * <p> This method implement a required XQService method.
     *
     * @param ctx The service context.
     * @exception XQServiceException Thrown in the event of a processing error.
     */
    public void service(XQServiceContext ctx) throws XQServiceException {
		m_xqLog.logDebug(m_logPrefix + "Service processing...");

        Configuration runtimeConfig = 
            loadConfigurationObject(ctx.getParameters(),false);
        if (runtimeConfig==null) {
            runtimeConfig = m_configObject;
        } 
        m_xstream.alias(runtimeConfig.getRoot(), Map.class);

        int runtimeOMPartOut = m_outputMessagePart;
        if (ctx.getParameters().containsParameter(OUTPUT_MESSAGE_PART, XQConstants.PARAM_STRING)) {
            runtimeOMPartOut = ctx.getParameters().getIntParameter(OUTPUT_MESSAGE_PART, 
                    XQConstants.PARAM_STRING); 
        }
        int runtimeOMPartIn = m_inputMessagePart;
        if (ctx.getParameters().containsParameter(OUTPUT_MESSAGE_PART, XQConstants.PARAM_STRING)) {
            runtimeOMPartIn = ctx.getParameters().getIntParameter(OUTPUT_MESSAGE_PART, 
                    XQConstants.PARAM_STRING); 
        }
         
        
		// Get the message.
        XQEnvelope env = null;
        XQMessageFactory factory = ctx.getMessageFactory();
        while (ctx.hasNextIncoming()) {
            env = ctx.getNextIncoming();
            
            if (env != null) {
                XQMessage msg = env.getMessage();
                XQMessage msgOut = factory.createMessage();
                try {
                    for (Iterator iter = msg.getHeaderNames(); iter.hasNext();) {
                        String element = (String) iter.next();
                        msgOut.setHeaderValue(element, msg.getHeaderValue(element));
                    }
                    int iPartCnt = msg.getPartCount();
                    for (int i=0; i< iPartCnt; i++) {
                        XQPart prt = msg.getPart(i);
                        msgOut.addPart(prt);                            
                        if (runtimeOMPartIn != -1 && runtimeOMPartIn != i ) {
                            continue;
                        }
                        String content = prt.getContent().toString();
                        StringBuilder regex = new StringBuilder("");
                        String first = runtimeConfig.getKeys().get(0).trim();
                        String last = runtimeConfig.getKeys().get(
                                runtimeConfig.getKeys().size()-1).trim();
                        boolean oneElement = runtimeConfig.getKeys().size() == 1;
                        
                        Map<String,String> elems = new HashMap<String,String>(); 
                        for (Iterator<String> iter = runtimeConfig.iterator(); iter
                                .hasNext();) {
                            String element = iter.next().trim();
                            if (oneElement || first.equals(element)) {
                                regex.append("(").append(element.trim())
                                    .append("=");
                            } else  {
                                regex.append("|\n").append(element.trim()).append("=");
                            } if (last.equals(element) || oneElement) {
                                regex.append(")");
                            }
                        }
                        m_xqLog.logDebug(" regex = "+regex.toString());
                        
                        StringBuilder sb = new StringBuilder(XML_HEADER);
                        //sb.append("\n<").append(runtimeConfig.getRoot())
                        //    .append(">");
                        String[] elements = content.split(regex.toString());
                        
                        //for (int j = 0; j < elements.length; j++) {
                        //    m_xqLog.logDebug("elements["+j+"]"+elements[j]);
                        //}
                        for (int j = 1; j < elements.length; j++) {
                            elems.put( runtimeConfig.get(j-1),elements[j].trim());
                            //sb.append("\n\t<").append(runtimeConfig.get(j-1))
                            //.append(">").append(elements[j].trim())
                            //.append("</").append(runtimeConfig.get(j-1))
                            //.append(">");
                        }
                        //sb.append("\n</").append(runtimeConfig.getRoot())
                        //.append(">\n\t");
                        sb.append(m_xstream.toXML(elems));
                        m_xqLog.logDebug(" map = "+elems.toString());
                        m_xqLog.logDebug(" xml = \n"+sb.toString());

                        
                        prt.setContent(sb.toString(), "text/xml");
                        if (runtimeOMPartOut == 0) {
                            msgOut.replacePart(prt, i);
                        } else {
                            prt.setContentId(prt.getContentId()+"-"+i);
                            msgOut.addPart(prt);                            
                        }
                    }
                    env.setMessage(msgOut);
                } catch (XQMessageException me) {
                    throw new XQServiceException("Exception accessing XQMessage: " + me.getMessage(), me);
                }

                // Pass message onto the outbox.
                Iterator addressList = env.getAddresses();
                if (addressList.hasNext())  {
                    // Add the message to the Outbox
                    ctx.addOutgoing(env);
                }
            }
        }
		m_xqLog.logDebug(m_logPrefix + "Service processed...");
    }
    
    /**
     * Clean up and get ready to destroy the service.
     *
     * <p> This method implement a required XQService method.
     */
    public void destroy() {
		m_xqLog.logInformation(m_logPrefix + "Destroying...");
		m_xqLog.logInformation(m_logPrefix + "Destroyed...");
    }
    
    /**
     * Called by the container on container start.
     *
     * <p> This method implement a required XQServiceEx method.
     */
	public void start() {
		m_xqLog.logInformation(m_logPrefix + "Starting...");
		m_xqLog.logInformation(m_logPrefix + "Started...");
	}
	
    /**
     * Called by the container on container stop.
     *
     * <p> This method implement a required XQServiceEx method.
     */
	public void stop() {
		m_xqLog.logInformation(m_logPrefix + "Stopping...");
		m_xqLog.logInformation(m_logPrefix + "Stopped...");
	}
	
	/**
     * Clean up and get ready to destroy the service.
     *
     */
	protected void setLogPrefix(XQParameters params) {
		String serviceName = params.getParameter(XQConstants.PARAM_SERVICE_NAME, XQConstants.PARAM_STRING);
		m_logPrefix = "[ " +  serviceName + " ]";
	}
	
    /**
     * Provide access to the service implemented version.
     *
     */
	protected String getVersion(){
		return s_major + "." + s_minor + ". build " + s_buildNumber;  
	}
	
    /**
     * Writes a standard service startup message to the log.
     */
    protected void writeStartupMessage(XQParameters params) {
        final StringBuffer buffer = new StringBuffer();
        String serviceTypeName = params.getParameter(XQConstants.SERVICE_PARAM_SERVICE_TYPE, XQConstants.PARAM_STRING);
        buffer.append("\n\n");
        buffer.append("\t\t " + serviceTypeName + "\n ");
        buffer.append("\t\t Version ");
        buffer.append(" " +getVersion());
        buffer.append("\n");
        m_xqLog.logInformation(buffer.toString());
	}
    
    /**
     * Writes parameters to log.
     */
    protected void writeParameters(XQParameters params) {

        if (m_xqLog.isDebugLoggingEnabled()) {
            final Map map = params.getAllInfo();
            final Iterator iter = map.values().iterator();
            
            while (iter.hasNext()) {
                final XQParameterInfo info = (XQParameterInfo) iter.next();
                
                if (info.getType() == XQConstants.PARAM_XML) {
                    m_xqLog.logDebug(m_logPrefix + "Parameter Name =  "
                            + info.getName());
                } else if (info.getType() == XQConstants.PARAM_STRING) {
                    m_xqLog.logDebug(m_logPrefix + "Parameter Name = "
                            + info.getName());
                }
                
                if (info.getRef() != null) {
                    m_xqLog.logDebug(m_logPrefix
                            + "Parameter Reference = " + info.getRef());
                    
                    // If this is too verbose
                    // /then a simple change from logInformation to logDebug
                    // will ensure file content is not displayed
                    // unless the logging level is set to debug for the ESB
                    // Container.
                    m_xqLog.logDebug(m_logPrefix
                            + "----Parameter Value Start--------");
                    m_xqLog.logDebug("\n" + info.getValue() + "\n");
                    m_xqLog.logDebug(m_logPrefix
                            + "----Parameter Value End--------");
                } else {
                    m_xqLog.logDebug(m_logPrefix + "Parameter Value = "
                            + info.getValue());
                }
            }
        }
    }
}