/**
 * 
 */
package com.sonicsw.pso.demo.util;

import java.io.File;
import java.io.FilenameFilter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

/**
 * @author Nick Rapoport - Copyright (c) - May 23, 2007 All rights reserved
 *
 */
public class ExtensionFilenameFilter implements FilenameFilter {
//    private static org.apache.log4j.Logger log = org.apache.log4j.Logger
//            .getLogger(ExtensionFilenameFilter.class);

    private List<Pattern> patterns = new ArrayList<Pattern>();
    private boolean caseSensitive = false;
    /**
     * 
     */
    public ExtensionFilenameFilter() {
        super();
     }
    public ExtensionFilenameFilter(List<String> extPatternList) 
    throws ConfigurationException {
        super();
        populatePatterns(extPatternList);
    }
    public ExtensionFilenameFilter(List<String> extPatternList,boolean matchCase) 
    throws ConfigurationException {
        this(extPatternList);
        setCaseSensitive(matchCase);
    }

    public ExtensionFilenameFilter(String extPatternsString) 
    throws ConfigurationException {
        this(Arrays.asList(extPatternsString.split(";")));
    }

    public ExtensionFilenameFilter(String extPatternsString,boolean matchCase) 
    throws ConfigurationException {
        this(Arrays.asList(extPatternsString.split(";")),matchCase);
    }
 
    protected void populatePatterns(String pStrings)
        throws ConfigurationException {
        populatePatterns(Arrays.asList(pStrings.split(";")));
    }

    protected void populatePatterns(List<String> paternStrings)
        throws ConfigurationException {
        try {
            for (Iterator<String> iter = paternStrings.iterator(); iter
                    .hasNext();) {
                String patern = iter.next();
                System.out.println("found pattern raw="+patern);
                patern = wildcardToRegex(patern);
                System.out.println("found pattern regex="+patern);
                if (isCaseSensitive()) {
                    patterns.add(Pattern.compile(patern.toLowerCase()));
                } else {
                    patterns.add(Pattern.compile(patern));
                }
            }
        }
        catch (java.util.regex.PatternSyntaxException pex) {
            throw new ConfigurationException(
                    "Error configuring pattern filter for "
                            + paternStrings.toString(), pex);
        }
    }
    protected static String wildcardToRegex(String wild) {
        if (wild == null) return null;
        
        StringBuffer buffer = new StringBuffer();
        
        char[] chars = wild.toCharArray();
        
        for (int i = 0; i < chars.length; ++i) {
            if (chars[i] == '*') {
                buffer.append(".*");
            } else if (chars[i] == '?') {
                buffer.append(".");
            } else if ("+()^$.{}[]|\\".indexOf(chars[i]) != -1) {
                //prefix all metacharacters with backslash
                buffer.append('\\').append(chars[i]); 
            } else {
                buffer.append(chars[i]);
            }
            if (i+1==chars.length && chars[i]!='$') {
                buffer.append('$');
            }
        }
        return buffer.toString();
    }

    /** 
     * @param file
     * @param name
     * @return true if the extension of this file name matches one of the included
     * extensions
     * @see java.io.FilenameFilter#accept(java.io.File, java.lang.String)
     */
    
    public boolean accept(@SuppressWarnings("unused")File file, String name) {
        List<Pattern> patterns = getPatterns();
        if (patterns.size()==0) {
            return true;
        }
        for (Iterator<Pattern> iter = patterns.iterator(); iter.hasNext();) {
            Pattern ptn = iter.next();
            if (isCaseSensitive()) {
                if (ptn.matcher(name).matches()) {
                    return true;
                }
            } else {
                if (ptn.matcher(name.toLowerCase()).matches()) {
                    return true;
                }
            }
        }
        return false;
    }
    
    /**
     * @param args
     */
    public static void main(String[] args) {
        // TODO Auto-generated method stub
        
    }

    /**
     * @param patterns the value to set patterns
     */
    public void setPatterns(List<Pattern> patterns) {
        this.patterns = patterns;
    }

    /**
     * @return the value of patterns
     */
    public List<Pattern> getPatterns() {
        return patterns;
    }
 
    /**
     * @return the value of caseSensitive
     */
    public boolean isCaseSensitive() {
        return caseSensitive;
    }

    /**
     * @param caseSensitive the value to set caseSensitive
     */
    public void setCaseSensitive(boolean caseSensitive) {
        this.caseSensitive = caseSensitive;
    }   
}
