using System;
using System.Xml.Serialization;
using System.Xml;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Sonic.Jms;
using global::sonicsw.pso.message.generator.Properties;
using Khendys.Controls;
using System.Diagnostics;
using System.IO;
using log4net;
using log4net.Config;
using System.Reflection;
using sonicsw.pso.utilities;
using System.Collections;
namespace sonicsw.pso.message.generator
{

    public partial class MessageGeneratorForm : Form, IJMSAccessDisplay 
    {
        #region Properties
        private static log4net.ILog log = log4net.LogManager.GetLogger(
            System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        XmlSerializer xmlSer
            = new XmlSerializer(typeof(MessageRoot));
        StringBuilder msgBuffer = new StringBuilder();
        System.IO.StringWriter sr;
        System.Collections.Hashtable props = new System.Collections.Hashtable();

        public System.Collections.Hashtable Props
        {
            get { return props; }
            set { props = value; }
        }

        JMSAccess jmsAccess;

        internal JMSAccess JmsAccess
        {
            get { return jmsAccess; }
            set { jmsAccess = value; }
        }
        private StreamWriter logFile = null;

        #endregion

        #region Constructors
        public MessageGeneratorForm()
        {
            InitializeComponent();

            sr = new System.IO.StringWriter(msgBuffer);
            Settings.Default.Init = true;
            InitializeDropDownLists();
            selectMessageSource();
            fillFactoryProperties();
            JMSException +=new EventHandler<JMSExceptionEventArgs>(this.ProcessJMSException);
            JMSConnectionState += new EventHandler<JMSConnectionStateEventArgs>(this.ProcessJMSConnectionState);
            JMSMessage += new EventHandler<JMSMessageEventArgs>(this.ProcessJMSMessage);
            WriteTrace += new EventHandler<WriteTraceEventArgs>(this.ProcessWriteTrace);
            TraceException += new EventHandler<TraceExceptionEventArgs> (this.ProcessTraceException);

            JmsAccess = new JMSAccess(this, Settings.Default);
            Settings.Default.Init = false;
            
            String logFolder = sfd_TraceResponseFile.InitialDirectory;
            String logFileName = Settings.Default.TraceResponseFile;
            logFile = validateLogFile(logFolder, logFileName);
            //String logFolder = sfd_TraceResponseFile.InitialDirectory()
            //if (
            //getFileReference(tb_TraceResponseFile, sfd_TraceResponseFile, "TraceFileName");
            //logFile = Utilities.CreateStreamWriter(tb_TraceResponseFile.Text);
        }

        private StreamWriter validateLogFile(String logFolder, String logFile)
        {
            StreamWriter sw = null;
            try
            {
                System.IO.FileInfo logFileInfo = null;
                bool fullPathFileName = false;
                if (logFile != null && logFile.Length > 1)
                {
                    if (logFile.StartsWith("\\\\") || logFile.StartsWith("//") || logFile.ToCharArray()[1] == ':')
                    {
                        fullPathFileName = true;
                    }
                }
                if (logFolder != null && !logFolder.Equals("") && !fullPathFileName) //&& 
                {
                    if (!Directory.Exists(logFolder))
                    {
                        Directory.CreateDirectory(logFolder);
                    }
                    logFileInfo = new FileInfo(logFolder + "\\" + logFile);
                }
                else
                {
                    logFileInfo = new FileInfo(logFile);
                }
                if (!logFileInfo.Exists)
                {
                    logFileInfo.Create();
                }
                sw = Utilities.CreateStreamWriter(logFileInfo.FullName);
            }
            catch (Exception ex)
            {
                ProcessException("MessageGeneratorForm()", ex);

            }
            return sw;
        }

        #endregion

        #region Initialize Screen
        private void InitializeDropDownLists()
        {
            cmb_FlowToDisk.DataSource = Enum.GetValues(typeof(FlowToDisk));
            cmb_FlowToDisk.SelectedItem =
                Enum.Parse(typeof(FlowToDisk),
                Settings.Default.FlowToDisk, true);
            cmb_AckMode.DataSource = Enum.GetValues(typeof(JMSAckMode));
            cmb_AckMode.SelectedItem =
                Enum.Parse(typeof(JMSAckMode),
                Settings.Default.AckMode, true);
            InitializeDestinationList(cmb_Destinations,Settings.Default.SelectedDest,Settings.Default.Destinations);
            InitializeDestinationList(cmb_ListenDestinations, Settings.Default.SelectedListenDest, Settings.Default.ListenDestinations);
        }

        private void InitializeDestinationList(ComboBox cb, string selectedDest, System.Collections.Specialized.StringCollection list)
        {
            cb.DataSource = list;
            if (list.Contains(selectedDest))
            {
                int pos = list.IndexOf(selectedDest);
                cb.SelectedIndex = pos;
            }
        }

        /// <summary>
        /// This Method populates the properies file that will be passed to the
        /// JmsAccess object and holds all the connection and session properties
        /// </summary>
        public void fillFactoryProperties()
        {
            setProperty("ClientID",this.tb_ClientID.Text);
            setProperty("ClientTransactionBufferSize",(long)
                this.nud_ClientTransactionBufferSize.Value);
            setProperty("ConnectID",this.tb_ConnectID.Text);
            setProperty("ConnectionURLs",this.tb_ConnectionURLs.Text);
            setProperty("DefaultPassword",this.tb_DefaultPassword.Text);
            setProperty("DefaultTxnBatchSize",(int)
                this.nud_DefaultTxnBatchSize.Value);
            setProperty("DefaultUser",this.tb_DefaultUser.Text);
            setProperty("DurableMessageOrder",
                this.cb_DurableMessageOrder.Checked);
            setProperty("FaultTolerant",this.cb_FaultTolerant.Checked);
            setProperty("FaultTolerantReconnectTimeout",(int)
                this.nud_FaultTolerantReconnectTimeout.Value);
            FlowToDisk flow = (FlowToDisk)this.cmb_FlowToDisk.SelectedItem;
            setProperty("FlowToDisk",(int)flow);

            //setProperty("HTTPCredentials(true,0,"object");
            //setProperty("HTTPProxyInfo("host",0,true);
            setProperty("InitialConnectTimeout",(int)nud_InitialConnectTimeout.Value);
            setProperty("LoadBalancing",cb_LoadBalancing.Checked);
            //setProperty("LoginSPI",factory.getLoginSPI());
            //setProperty("LoginSPILibrary",factory.getLoginSPI());
            setProperty("MonitorInterval",(int)nud_MonitorInterval.Value);
            setProperty("PersistentDelivery",cb_PersistentDelivery.Checked);
            setProperty("PingInterval",(long)nud_PingInterval.Value);
            setProperty("PrefetchCount",(int)nud_PrefetchCount.Value);
            setProperty("PrefetchThreshold",(int)nud_PrefetchThreshold.Value);
            setProperty("SelectorAtBroker",cb_SelectorAtBroker.Checked);
            setProperty("Sequential", cb_Sequential.Checked);
            setProperty("Sequential", cb_Sequential.Checked);
            setProperty("Queue", cb_Queue.Checked);
            setProperty("ListenQueue", cb_ListenQueue.Checked);
            setProperty("ListenDestChanged", true);
            setProperty("DestChanged", true);
            //setProperty("SSLCertificate(0, "certStoreName", 0, "certStoreInfo");
            //setProperty("SSLCommonName();
        }

           
        private void MessageGeneratorForm_Load(object sender, EventArgs e)
        {

        }
       
        /// <summary>
        /// Use the OpenFileDialog to pick a file name and place it in the 
        /// TextBox and the properties Collection under the propName name 
        /// </summary>
        /// <param name="tb">TexBox to put the file name into</param>
        /// <param name="fd">FileDialog to use</param>
        /// <param name="propName">the string that will be the property key</param>
        /// <param name="propName">the string that will be the property key</param>
        private void getFileReference(TextBox tb, FileDialog fd, string propName)
        {
            DialogResult result = fd.ShowDialog(this);
            if (result == DialogResult.OK)
            {
                String fName = fd.FileName;
                setProperty(propName, fName);
                tb.Text = fName;
            }
        }
        /// <summary>
        /// Adds/replaces key value pair in tthe Hashtable used in JMSAccess 
        /// Object
        /// </summary>
        /// <param name="key"></param>
        /// <param name="valueObj"></param>
        public void setProperty(String key, Object valueObj)
        {
            if (this.props.ContainsKey(key))
            {
                this.props[key] = valueObj;
            }
            else
            {
                this.props.Add(key, valueObj);
            }
        }


        #endregion

        #region JMS Access Methods  
        private void sendMessage()
        {
            if (tssl_connectStatus.Text == "Disconnected")
            {
                tssl_StatusMessage.Text = "Please Connect Prior To Sending";
            }
            else
            {
                MessageRoot msgObj;
                msgObj = createMessageRoot(msgBuffer);
                if (msgObj != null)
                {
                    JmsAccess.sendMessage(msgObj, msgBuffer);
                    setMessageSentStatus();
                }
                else
                {
                    WriteTraceMessage("Error Sending Message", "invalid File", Color.Red);
                }
            }
        }

        private MessageRoot createMessageRoot(StringBuilder msgBuffer)
        {
            MessageRoot msgObj;
            msgBuffer.Remove(0, msgBuffer.Length);
            tssl_StatusMessage.Text = "Sending ...";
            long ticks = DateTime.Now.Ticks;
            if (rb_Text.Checked)
            {
                msgObj = new MessageRoot(tb_Key.Text.Trim() + "-" + ticks, tb_Message.Text);
            }
            else
            {   //check to see if file name is same as when it was selected
                if (!ofd_GetMessageFile.FileName.Equals(tb_FileName.Text)
                    || ofd_GetMessageFile.FileName.Equals(ofd_GetMessageFile.SafeFileName))
                {
                    getFileReference(tb_FileName, ofd_GetMessageFile, "FileName");
                }
                msgObj = getMessageRootFromFile(ticks);
            }
            if (msgObj != null)
            {
                xmlSer.Serialize(sr, msgObj);
                sr.Flush();
                msgBuffer.Replace("utf-16", "utf-8");
            }
            return msgObj;
        }

        private MessageRoot getMessageRootFromFile(long ticks)
        {
            MessageRoot msgObj =null;
            String key = ofd_GetMessageFile.SafeFileName+"-"+ticks;
            String data = "File empty or does not exist";
            System.IO.Stream s = null;
            try { s = ofd_GetMessageFile.OpenFile(); }
            catch
            {
                getFileReference(tb_FileName, ofd_GetMessageFile, "FileName");
                key = ofd_GetMessageFile.SafeFileName + "-" + ticks;
                try { s = ofd_GetMessageFile.OpenFile(); }
                catch
                {
                }
            }
            if (s != null)
            {
                System.IO.StreamReader streamReader =
                    new System.IO.StreamReader(s);
                data = streamReader.ReadToEnd();
                data = data.Replace('\'', '"');
                msgObj = new MessageRoot(key, data);
            }
            return msgObj;
        }

        private void disconnect(Button btn)
        {
            tssl_StatusMessage.Text = "";
            JmsAccess.cleanUp();
            fillFactoryProperties();
            tssl_connectStatus.Text = "Disconnected";
            cmb_ListenDestinations.Enabled = true;
            cb_ListenQueue.Enabled = true;
            tssl_connectStatus.ForeColor = System.Drawing.Color.DarkRed;
            if (btn.Equals(btn_Connect)) { btn.Text = "Connect"; }
        }

        private void connect(Button btn)
        {
            tssl_StatusMessage.Text = "";
            JmsAccess.cleanUp();
            fillFactoryProperties();
            tssl_connectStatus.Text = "Disconnected";
            tssl_connectStatus.ForeColor = System.Drawing.Color.DarkRed;
            try
            {
                JmsAccess.connect();
                tssl_connectStatus.Text = "Connected";
                tssl_connectStatus.ForeColor = System.Drawing.Color.DarkBlue;
                if (btn.Equals(btn_Connect)) { btn.Text = "Disconnect"; }
                cmb_ListenDestinations.Enabled = false;
                cb_ListenQueue.Enabled = false;

            }
            catch (Exception ex)
            {
                //tssl_StatusMessage.Text = ex.GetType().Name +  ":" + ex.Message;
                RaiseTraceException(JmsAccess,new TraceExceptionEventArgs("connect(...)", ex));
            }
        }

        #endregion

        #region Event Methods
        private void SelectedIndexChanged(object sender, EventArgs e)
        {
            if (sender == cmb_FlowToDisk)
            {
                FlowToDisk currentFlowToDisk = (FlowToDisk)cmb_FlowToDisk.SelectedItem;
                setProperty("FlowToDisk", currentFlowToDisk);

                String val = Enum.GetName(typeof(FlowToDisk), currentFlowToDisk);
                if (!Settings.Default["FlowToDisk"].Equals(val) && !Settings.Default.Init)
                {
                    Settings.Default["FlowToDisk"] = val;
                    Settings.Default.Save();
                }
            }
            else if (sender == cmb_AckMode)
            {
                JMSAckMode currentAckMode = (JMSAckMode)cmb_AckMode.SelectedItem;
                String val = Enum.GetName(typeof(JMSAckMode), currentAckMode);
                setProperty("AckMode", currentAckMode);
                if (!Settings.Default["AckMode"].Equals(val) && !Settings.Default.Init)
                {
                    Settings.Default["AckMode"] = val;
                    Settings.Default.Save();
                }
            }
            else if (sender == cmb_Destinations)
            {
                processDestinationChange(cmb_Destinations, Settings.Default.SelectedDest
                    , "SelectedDest", "DestChanged", Settings.Default.Destinations);
                //destChanged = true;
            }                                                    
            else if (sender == cmb_ListenDestinations)
            {
                processDestinationChange(cmb_ListenDestinations, Settings.Default.SelectedListenDest
                    , "SelectedListenDest","ListenDestChanged",Settings.Default.ListenDestinations );
                //destChanged = true;
            }

        }

        private void processDestinationChange(ComboBox cb, string oldSelection, string selectionName, string selectionChangedName, System.Collections.Specialized.StringCollection list)
        {
            string val = (string)cb.SelectedItem;
            if (val == null)
            {
                val = cb.Text;
            }
            setProperty(selectionName, val);
            setProperty(selectionChangedName, true);
            if (!Settings.Default.Init)
            {
                if (!oldSelection.Equals(val))
                {
                    Settings.Default.Init = true;
                    Settings.Default[selectionName] = val;

                }
                if (!list.Contains(val))
                {
                    list.Add(val);
                    Settings.Default.Init = true;
                    int loc = list.IndexOf(val);
                    cb.DataSource = null;
                    cb.DataSource =
                        list;
                    cb.SelectedIndex = loc;
                }
                if (Settings.Default.Init)
                {
                    Settings.Default.Init = false;
                    Settings.Default.Save();
                }
            }
        }
        
        private void ValueChanged(object sender, EventArgs e)
        {
            if (sender is NumericUpDown)
            {
                NumericUpDown nud = (NumericUpDown)sender;

                System.Windows.Forms.Binding binding = nud.DataBindings["Value"];
                object val = nud.Value;
                if (binding != null && !((decimal)val).Equals(-1d))
                {
                    updateSettings(binding, val);
                }

            }
            else if (sender is CheckBox)
            {
                CheckBox cb = (CheckBox)sender;
                System.Windows.Forms.Binding binding = cb.DataBindings["Checked"];
                object val = cb.Checked;
                if (binding != null)
                {
                    updateSettings(binding, val);
                }
                if (cb == cb_Queue)
                {
                    setProperty("DestChanged", true);
                    setProperty("Queue", cb_Queue.Checked);
                    //destChanged = true;
                }
                if (cb == cb_ListenQueue)
                {
                    setProperty("ListenDestChanged", true);
                    setProperty("ListenQueue", cb_ListenQueue.Checked);
                    //destChanged = true;
                }

            }
            else if (sender is TextBox)
            {
                TextBox tb = (TextBox)sender;
                System.Windows.Forms.Binding binding = tb.DataBindings["Text"];
                object val = tb.Text;
                if (binding != null)
                {
                    updateSettings(binding, val);
                }
            }

        }

        private void updateSettings(System.Windows.Forms.Binding binding, object val)
        {
            //new Binding(
            object ds = binding.DataSource;
            if (ds is Settings)
            {
                String prop_name = binding.BindingMemberInfo.BindingField;
                setProperty(prop_name, val);
                if (!((Settings)ds)[prop_name].Equals(val))
                {
                    ((Settings)ds)[prop_name] = val;
                    ((Settings)ds).Save();
                }
            }
        }

        private void test_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (sender is ComboBox)
            {
                if (e.KeyChar.Equals('\n'))
                {
                    SelectedIndexChanged(sender, e);
                }

            }
        }

        private void btn_Click(object sender, EventArgs e)
        {
            if (sender is Button)
            {
                Button btn = (Button)sender;
                if (btn.Text == "Connect")
                {
                    connect(btn);

                }
                else if ((btn.Text == "Disconnect"))
                {
                    disconnect(btn);
                }
                else if (btn == btn_Submit)
                {
                    sendMessage();

                }
                else if ((btn == btn_SetDefaults))
                {
                    JmsAccess.loadFactoryDefaults();
                }
                else if ((btn.Text == ". . ."))
                {
                    if (btn == btn_fileOpen)
                    {
                        getFileReference(tb_FileName, ofd_GetMessageFile, "FileName");
                    } else if (btn == btn_TraceFileOpen) {
                        String holdName = tb_TraceResponseFile.Text;
                        sfd_TraceResponseFile.FileName = tb_TraceResponseFile.Text;
                        getFileReference(tb_TraceResponseFile, sfd_TraceResponseFile, "TraceResponseFile");
                        if (!sfd_TraceResponseFile.FileName.Equals(holdName)) {
                            logFile = validateLogFile("", tb_TraceResponseFile.Text);
                        }

                    }

                }
                else if ((btn == btn_ClearStatus))
                {
                    rtb_Status.Clear();
                }
                else if ((btn.Text == "Exit"))
                {
                    disconnect(btn);
                    this.Close();
                    Application.Exit();
                }
            }
        }
        
        private void setMessageSentStatus()
        {
            tssl_StatusMessage.Text = "Sent Message ...";
            int cntr = int.Parse(tssl_cntr.Text);
            cntr++;
            tssl_cntr.Text = cntr.ToString();
            this.statusStrip1.ResumeLayout(false);
            this.statusStrip1.PerformLayout();
        }

        private void keyEvent(object sender, KeyEventArgs e)
        {
            if (sender is ComboBox)
            {
                if (e.KeyCode == Keys.Enter)
                {
                    SelectedIndexChanged(sender, e);
                }
            }
        }

        private void rb_CheckChanged(object sender, EventArgs e)
        {
            selectMessageSource();
        }

        private void selectMessageSource()
        {
            tb_FileName.Enabled = rb_File.Checked;
            btn_fileOpen.Enabled = rb_File.Checked;
            tb_Key.Enabled = rb_Text.Checked;
            tb_Message.Enabled = rb_Text.Checked;
            setProperty("TextChecked", rb_Text.Checked);
            setProperty("FileChecked", rb_File.Checked);
        }

        #endregion

        #region Add Text To RTF Control
        private void AddFormated(string valueIn, System.Drawing.Color color)
        {
            string foreground = color.Name;
            string background = rtb_Status.BackColor.Name;
            if (rtb_Status.BackColor.IsSystemColor)
            {
                background = "White";
            }
            RtfColor rtfColor = (RtfColor)Enum.Parse(typeof(RtfColor), foreground, true);
            RtfColor rtfBackColor = (RtfColor)Enum.Parse(typeof(RtfColor), background, true);
            this.rtb_Status.AppendTextAsRtf(valueIn, rtb_Status.Font, rtfColor, rtfBackColor);
            if (cb_StatusBullets.Checked)
            {
                int start = this.rtb_Status.Text.Length;
                //Debug.WriteLine("Start = " + start);
                int end = this.rtb_Status.Text.Length;
                //Debug.WriteLine("end = "+end);
                this.rtb_Status.Select(start, end);
                this.rtb_Status.SelectionBullet = true;
                //this.rtb_Status.SelectionColor = color;
            }
            this.rtb_Status.Select(0, 0);
            trimLog();

        }

        protected void trimLog()
        {
            try
            {
                if (this.cb_TraceResponseMessages.Checked)
                {
                    int maxLength = Convert.ToInt32(this.nud_MaxLogSize.Value) * 1024;
                    int length = this.rtb_Status.TextLength;
                    if (length > maxLength)
                    {
                        int offset = length - maxLength;
                        this.rtb_Status.Select(0, offset);
                        this.rtb_Status.Cut();
                    }

                }
            }
            catch (Exception ex)
            {
                ProcessException("trimLog()", ex);
            }
        }
       
        private void ProcessException(string prefix, Exception ex)
        {
            WriteTraceMessage(prefix, "Exception:" + ex.GetType().Name, System.Drawing.Color.Red);
            WriteTraceMessage(prefix, "Exception:" + ex.Message, System.Drawing.Color.Red);
            WriteTraceMessage(prefix, "Exception:" + ex.StackTrace, System.Drawing.Color.Red);
        }
        
        private void WriteTraceMessage(string prefix, string data, System.Drawing.Color color)
        {
            String tsString = Utilities.TimeStampString(prefix, string.Format("{0}\r\n", data));
            if (this.cb_TraceResponseMessagesToFile.Checked)
            {
               Utilities.WriteToFile(logFile, tsString);
            }
            if (this.cb_TraceResponseMessages.Checked)
            {
                tsString = string.Format("\r\n{0}\r\n", tsString);
                AddFormated(tsString, color);
            }

        }
        #endregion

        #region IJMSAccessDisplay Members

        public void ProcessJMSException(Object sender, JMSExceptionEventArgs e)
        {
            ProcessException("JMSExceptionEventHandler(...)", e.JMSException);
            disconnect(btn_Connect);
        }

        public void ProcessJMSConnectionState(Object sender, JMSConnectionStateEventArgs e)
        {
            Color color = Color.Blue;

            color = e.State == JmsConnectionStates.ACTIVE ?
                Color.Green : color;

            if (e.State == JmsConnectionStates.CLOSED || e.State == JmsConnectionStates.FAILED)
            {
                color = Color.Red;
                disconnect(btn_Connect);
            }

            WriteTraceMessage("ConnectionStateHandler(...)",
                "ConnectionState changed : "+e.getStateName(),
                color);
        }

        public void ProcessJMSMessage(Object sender, JMSMessageEventArgs e)
        {
            Color color = Color.Navy;
            StringBuilder xml = new StringBuilder();
            xml.Append("\r\n<Message>");
            try
            {
                Dictionary<string, object>
                    jmsHeaders = JMSAccess.GetJMSHeaderProps(e.Message);

                jmsHeaders = JMSAccess.GetUserProps(e.Message, jmsHeaders);

                xml.Append("\r\n\t<MessageHeader>");
                foreach(string key in jmsHeaders.Keys) 
                {
                    xml.Append("\r\n\t\t<Key>");
                    xml.Append(key);
                    xml.Append("</Key>");
                    xml.Append("<Value>");
                    Object o = jmsHeaders[key];
                    if (o != null)
                    {
                        if (o is Sonic.Jms.Queue)
                        {
                            xml.Append("QUEUE:");
                            xml.Append(((Sonic.Jms.Queue)o).getQueueName());
                        }
                        else if (o is Sonic.Jms.Topic)
                        {
                            xml.Append("TOPIC:");
                            xml.Append(((Sonic.Jms.Topic)o).getTopicName());
                        }
                        else
                        {
                            xml.Append(jmsHeaders[key]);
                        }
                    }
                    xml.Append("</Value>");
                }
                xml.Append("\r\n\t</MessageHeader>");
                if (e.Message is Sonic.Client.Jms.Impl.Ext.XMLMessage)
                {
                    xml.Append("\r\n\t<XmlMessage>");
                    xml.Append("\r\n\t\t<![CDATA[");
                    xml.Append(((Sonic.Client.Jms.Impl.Ext.XMLMessage)e.Message).getText());
                    xml.Append("]]>");
                    xml.Append("\r\n\t</XmlMessage>");
                }
                else if (e.Message is Sonic.Jms.TextMessage)
                {
                    xml.Append("\r\n\t<TextMessage>");
                    xml.Append("\r\n\t\t<![CDATA[");
                    xml.Append(((Sonic.Jms.TextMessage)e.Message).getText());
                    xml.Append("]]>");
                    xml.Append("\r\n\t</TextMessage>");
                }
                else if (e.Message is Sonic.Client.Jms.Impl.Ext.MultipartMessage)
                {
                    Sonic.Client.Jms.Impl.Ext.MultipartMessage multiMessage =
                        (Sonic.Client.Jms.Impl.Ext.MultipartMessage)e.Message;
                    xml.Append("\r\n\t<Multipart>");
                    int cnt = multiMessage.getPartCount();
                    for (int i = 0; i < cnt; i++) {
                        Sonic.Jms.Ext.Part part =
                        multiMessage.getPart(i);
                        Sonic.Jms.Ext.Header header = part.getHeader();
                        String contentId = header.getContentId();
                        xml.Append("\r\n\t\t<Header>");
                        string name = "";
                        for (IEnumerator ie = header.getHeaderFieldNames(); ie.MoveNext(); name = ie.Current.ToString())
                        {
                            xml.Append("\r\n\t\t\t<Name>");
                            xml.Append(name);
                            xml.Append("</Name>");
                            xml.Append("<Value>");
                            xml.Append(header.getHeaderField(name,""));
                            xml.Append("</Value>");
                        
                        }
                        xml.Append("\r\n\t\t</Header>");
                        String content = part.getContent().ToString();
                        xml.Append("<\r\n\t\tpart id='" + i + "' contentId='" + contentId + "'>");
                        xml.Append("<![CDATA[");
                        xml.Append(content);
                        xml.Append("]]>");
                        xml.Append("\r\n\t\t</part>");
                    }
                    xml.Append("\r\n\t</Multipart>");
                }
                //XmlSerializer xmlMsg = new XmlSerializer(e.Message.GetType());
                //StringBuilder msgBuffer = new StringBuilder();
                //StringWriter msgWriter = new StringWriter(msgBuffer);
                //xmlMsg.Serialize(msgWriter, e.Message);
                //msgWriter.Flush();
                //msgBuffer.Replace("UTF-16", "UTF-8");
            }
            catch (Exception ex)
            {
                ProcessException("ProcessJMSMessage(..)", ex);
            }

            xml.Append("\r\n</Message>");
            WriteTraceMessage("ProcessJMSMessage(...)",
                "JMSMessage recieved : " + xml,
                color);
        }


            
        

        public event EventHandler<JMSExceptionEventArgs> JMSException;
        protected virtual void OnJMSException(Object sender, JMSExceptionEventArgs e)
        {
            if (JMSException != null)
            {
                JMSException(sender, e);
            }
        }

        public event EventHandler<JMSConnectionStateEventArgs> JMSConnectionState;
        protected virtual void OnJMSConnectionState(Object sender, JMSConnectionStateEventArgs e)
        {
            if (JMSConnectionState != null)
            {
                JMSConnectionState(sender, e);
            }
        }

        public event EventHandler<JMSMessageEventArgs> JMSMessage;
        protected virtual void OnJMSMessage(Object sender, JMSMessageEventArgs e)
        {
            if (JMSMessage != null)
            {
                JMSMessage(sender, e);
            }

        }

        public void RaiseJMSMessage(Object sender, JMSMessageEventArgs e)
        {
            if (this.InvokeRequired)
            {
                BeginInvoke(new RaiseJMSMessageDel(RaiseJMSMessage), new object[] { sender, e });
            }
            else
            {
                this.OnJMSMessage(sender, e);
            }
        }

        public void RaiseJMSException(Object sender, JMSExceptionEventArgs e)
        {
            if (this.InvokeRequired)
            {
                BeginInvoke(new RaiseJMSExceptionDel(RaiseJMSException), new object[] { sender, e });
            }
            else
            {
                this.OnJMSException(sender, e);
            }
        }

        public void RaiseJMSConnectionState(Object sender, JMSConnectionStateEventArgs e)
        {
            if (this.InvokeRequired)
            {
                BeginInvoke(new RaiseJMSConnectionStateDel(RaiseJMSConnectionState), new object[] { sender, e });
            }
            else
            {
                this.OnJMSConnectionState(sender, e);
            }
        }
        

        #endregion

        #region IProcessTrace Members

        public void ProcessWriteTrace(object sender, WriteTraceEventArgs e)
        {
            WriteTraceMessage(e.Prefix, e.Data, e.Color);
        }

        public void ProcessTraceException(object sender, TraceExceptionEventArgs e)
        {
            ProcessException(e.Prefix, e.Exception);
        }

        public void RaiseWriteTrace(object sender, WriteTraceEventArgs e)
        {
            if (this.InvokeRequired)
            {
                BeginInvoke(new RaiseWriteTraceDel(RaiseWriteTrace),new object[] {sender, e});
            }
            else
            {
                this.OnWriteTrace(sender, e);
            }
        }

        public void RaiseTraceException(object sender, TraceExceptionEventArgs e)
        {
            if (this.InvokeRequired)
            {
                BeginInvoke(new RaiseTraceExceptionDel(RaiseTraceException),new object[] {sender, e});
            }
            else
            {
                this.OnTraceException(sender, e);
            }
        }

        public event EventHandler<WriteTraceEventArgs> WriteTrace;
        protected virtual void OnWriteTrace(Object sender, WriteTraceEventArgs e)
        {
            if (WriteTrace != null)
            {
                WriteTrace(sender, e);
            }

        }

        public event EventHandler<TraceExceptionEventArgs> TraceException;
        protected virtual void OnTraceException(Object sender, TraceExceptionEventArgs e)
        {
            if (TraceException != null)
            {
                TraceException(sender, e);
            }

        }

        #endregion
    }
}