using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Sonic.Jms;
using Constants = Sonic.Jms.Ext.Constants;

namespace sonicsw.pso.sqlserver.procedures
{

    public abstract class AbstractJMSComm : IDisposable
    {
        private string destName = null;
        bool newDest = true;
        public String DestName
        {
            get { return destName; }
            set
            {
                newDest = true; 
                destName = value; 
            }
        }
        private JMSDestType destType = JMSDestType.TOPIC;
        public JMSDestType DestType
        {
            get { return destType; }
            set 
            {
                newDest = true; 
                destType = value;
            }
        }

 
        private bool isConsumer = false;
        public bool IsConsumer
        {
            get { return isConsumer; }
            set { isConsumer = value; }
        }
        Destination destination = null;
        public Destination Destination
        {
            get 
            {
                Sonic.Jms.Session session = isConsumer ? ConsumerSession : ProducerSession;
                if ((destination == null || newDest) && session != null)
                {
                    if (DestType.Equals(JMSDestType.TOPIC))
                    {
                        destination = session.createTopic(DestName);
                    }
                    else
                    {
                        destination = session.createQueue(DestName);
                    }
                    newDest = false;
                }
                return destination;
                
             }
             set { destination = value; }
        }

        private bool ftConnection = true;
        public bool FTConnection
        {
            get { return ftConnection; }
            set { ftConnection = value; }
        }

        private Sonic.Jms.ConnectionFactory connFactory = null;
        public Sonic.Jms.ConnectionFactory ConnFactory
        {
            get { return connFactory; }
            set { connFactory = value; }
        }
        private Sonic.Jms.Connection connection = null;
        public Sonic.Jms.Connection Connection
        {
            get { return connection; }
            set { connection = value; }
        }
        private Sonic.Jms.Session producerSession = null;
        public Sonic.Jms.Session ProducerSession
        {
            get { return producerSession; }
            set { producerSession = value; }
        }
        private Sonic.Jms.Session consumerSession = null;
        public Sonic.Jms.Session ConsumerSession
        {
            get { return consumerSession; }
            set { consumerSession = value; }
        }

        private Sonic.Jms.MessageProducer producer = null;
        public Sonic.Jms.MessageProducer Producer
        {
            get { return producer; }
            set { producer = value; }
        }
        private Sonic.Jms.MessageConsumer consumer = null;
        public Sonic.Jms.MessageConsumer Consumer
        {
            get { return consumer; }
            set { consumer = value; }
        }


        public AbstractJMSComm() { }

        public AbstractJMSComm(string destNameIn)
        {
            destName = destNameIn;
        }

        public AbstractJMSComm(string destNameIn, JMSDestType typeIn)
        {
            destName = destNameIn;
            destType = typeIn;
        }

        public virtual string configure(string connectionURLs, string defaultUserName, string defaultPassword)
        {
            return configure(connectionURLs, defaultUserName, defaultPassword, null, JMSDestType.TOPIC);
        }

        public virtual string configure(string connectionURLs, string defaultUserName, string defaultPassword, string destNameIn, JMSDestType destTypeIn)
        {
            return configure(connectionURLs, defaultUserName, defaultPassword, destNameIn, destTypeIn, "");
        }

        public virtual string configure(string connectionURLs, string defaultUserName, string defaultPassword, string destNameIn, JMSDestType destTypeIn, string forwardQueue)
        {
            try
            {
                if (destNameIn != null && !destNameIn.Equals(""))
                {
                    DestName = destNameIn;
                    DestType = destTypeIn;
                }
                //TODO: check to see if we already connected to the same "connectionURLs" then skip most of this code
                connFactory = new Sonic.Jms.Cf.Impl.ConnectionFactory();
                ((Sonic.Jms.Cf.Impl.ConnectionFactory)connFactory).setConnectionURLs(connectionURLs);
                ((Sonic.Jms.Cf.Impl.ConnectionFactory)connFactory).setDefaultUser(defaultUserName);
                ((Sonic.Jms.Cf.Impl.ConnectionFactory)connFactory).setDefaultPassword(defaultPassword);
                ((Sonic.Jms.Cf.Impl.ConnectionFactory)connFactory).setFaultTolerant(ftConnection);
                // TODO: add any custom settings here

                connection = ConnFactory.createConnection();

                producerSession = connection.createSession(false, Sonic.Jms.SessionMode.AUTO_ACKNOWLEDGE);
                consumerSession = connection.createSession(false, Sonic.Jms.SessionMode.AUTO_ACKNOWLEDGE);

            }
            catch (JMSException ex)
            {
                StringBuilder sb = new StringBuilder("ERROR connecting to the ").Append(connectionURLs.ToString())
                    .Append(" broker, for user '").Append(defaultUserName.ToString())
                    .Append("' ").Append(ex.ToString());
                return sb.ToString();
            }
            return "Configured successfully";
        }


        #region IDisposable Members

        public virtual void Dispose()
        {
            if (consumer != null)
            {
                try { consumer.close(); }
                catch { }
                finally { consumer = null; }
            }
            if (producer != null)
            {
                try { producer.close(); }
                catch { }
                finally { producer = null; }
            }
            if (producerSession != null)
            {
                try { producerSession.close(); }
                catch { }
                finally { producerSession = null; }
            }
            if (consumerSession != null)
            {
                try { consumerSession.close(); }
                catch { }
                finally { consumerSession = null; }
            }
            if (connection != null)
            {
                try { connection.close(); }
                catch { }
                finally { connection = null; }
            }
        }

        #endregion
    }


}
