using System;
using System.Collections.Generic;
using System.Text;
using Sonic.Jms;
using Constants = Sonic.Jms.Ext.Constants;
using System.Collections;

namespace sonicsw.pso.sqlserver.procedures
{
    public class Consumer : AbstractJMSComm
    {
        public Consumer() : base() 
        {
            IsConsumer = true;
        }

        public Consumer(string dest)
            : base(dest)
        {
            IsConsumer = true;
        }

        public Consumer(string dest, JMSDestType dType)
            : base(dest, dType)
        {
            IsConsumer = true;
        }
        private bool forwardMessages = false;
        public bool ForwardMessages
        {
            get { return forwardMessages; }
            set { forwardMessages = value; }
        }

        private string forwardDestName = null;
        bool newForwardDest = true;
        public String ForwardDestName
        {
            get { return forwardDestName; }
            set
            {
                newForwardDest = true;
                forwardDestName = value;
            }
        }
        private JMSDestType forwardDestType = JMSDestType.QUEUE;
        public JMSDestType ForwardDestType
        {
            get { return forwardDestType; }
            set
            {
                newForwardDest = true;
                forwardDestType = value;
            }
        }

        Destination forwardDestination = null;
        public Destination ForwardDestination
        {
            get
            {
                Sonic.Jms.Session session = ConsumerSession;
                if ((forwardDestination == null || newForwardDest) && session != null)
                {
                    if (ForwardDestType.Equals(JMSDestType.TOPIC))
                    {
                        forwardDestination = session.createTopic(ForwardDestName);
                    }
                    else
                    {
                        forwardDestination = session.createQueue(ForwardDestName);
                    }
                    newForwardDest = false;
                }
                return forwardDestination;

            }
            set { forwardDestination = value; }
        }

        public override string configure(string brokerURL, string defaultUserName, string defaultPassword)
        {
            return configure(brokerURL, defaultUserName, defaultPassword, "", JMSDestType.TOPIC, "");
        }
        public override string configure(string brokerURL, string defaultUserName, string defaultPassword, string destNameIn, JMSDestType destTypeIn)
        {
            return configure(brokerURL, defaultUserName, defaultPassword, destNameIn, destTypeIn, "");
        }
        public override string configure(string brokerURL, string defaultUserName, string defaultPassword, string destNameIn, JMSDestType destTypeIn, string forwardQueue)
        {
            string status = base.configure(brokerURL, defaultUserName, defaultPassword, destNameIn, destTypeIn,"");

            if (DestType==JMSDestType.QUEUE && forwardQueue != null && !forwardQueue.Equals(""))
            {
                ForwardDestName = forwardQueue;
                forwardMessages = true;
            }
            else
            {
                ForwardDestName = "";
                forwardMessages = false; 
            }
            if (status.StartsWith("ERROR"))
            {
                return status;
            }
            try
            {
                if (ForwardMessages && ConsumerSession.getAcknowledgeMode() != Sonic.Jms.Ext.SessionMode.SINGLE_MESSAGE_ACKNOWLEDGE)
                {
                    ConsumerSession = Connection.createSession(false, Sonic.Jms.Ext.SessionMode.SINGLE_MESSAGE_ACKNOWLEDGE);
                }
                Consumer = ConsumerSession.createConsumer(Destination);
            }
            catch (JMSException ex)
            {
                status = "ERROR Creating MessageProducer for " + Enum.GetName(DestType.GetType(), DestType) + " named '" + DestName + "'" + ex.ToString();
            }
            return status;
        }

        public string ReceiveNoWait()
        {
            return Receive(0L);
        }

        public string Receive()
        {
            return Receive(-1L);
        }

        public string Receive(long timeout)
        {
            try
            {
                Message msg = null;
                if (Consumer != null)
                {
                    lock (Connection)
                    {
                        Connection.start();
                        if (timeout > 0)
                        {
                            msg = Consumer.receive(timeout);
                        }
                        else if (timeout == 0)
                        {
                            msg = Consumer.receiveNoWait();
                        }
                        else
                        {
                            msg = Consumer.receive();
                        }
                        Connection.stop();
                    }
                }
                else
                { 
                    return "ERROR receiving: Consumer not initialized"; 
                }
                //TextMessage txtMsg = msg as TextMessage;
                //if (txtMsg != null)
                //{
                //    Sonic.Client.Mgram.V26.Mgram mgrm = new Mgram();
                //    ((Sonic.Client.Jms.Impl.TextMessage)txtMsg).convertToMgram(mgrm);
                //    Message msg = Sonic.Client.Jms.Impl.TextMessage.createMessage(mgrm);
                //}

                //IEnumerator propsEnum = msg.getPropertyNames();
                //foreach(String name in propsEnum) {
                //    object val = msg.getObjectProperty(name);
                //    //...
                //}
                
                if (msg != null)
                {
                    if (ForwardMessages)
                    {
                        //TODO: Add a more message processing 
                        ((Sonic.Client.Jms.Impl.Message)msg).acknowledgeAndForward(ForwardDestination);
                    }
                    else if (ConsumerSession.getAcknowledgeMode() != Sonic.Jms.SessionMode.AUTO_ACKNOWLEDGE &&
                        ConsumerSession.getAcknowledgeMode() != Sonic.Jms.SessionMode.DUPS_OK_ACKNOWLEDGE)
                    {
                        //TODO: Add a more ack and transaction processing 
                        msg.acknowledge();
                        if (ConsumerSession.getAcknowledgeMode() == Sonic.Jms.SessionMode.SESSION_TRANSACTED)
                        {
                            ConsumerSession.commit();
                        }
                    }
                    //TODO: Add a more sofisticated message resolution: i.e. BytesMessage, MapMessage, etc 
                    return ((TextMessage)msg).getText();
                }
            }
            catch (JMSException ex)
            {
                StringBuilder sb = new StringBuilder("ERROR receiving from '")
                    .Append(DestName).Append("' :").Append(ex.ToString()); 
                return sb.ToString();
            }
            return "No Messages";
        }

    }

}
