using System;
using System.Collections.Generic;
using System.Text;
using Sonic.Jms;
//using Sonic.Jms.Ext;
using System.Collections;
using System.Collections.Specialized;
using Constants = Sonic.Jms.Ext.Constants;

namespace sonicsw.pso.sqlserver.procedures
{

    public class Producer : AbstractJMSComm
    {
        public Producer() : base() { }

        public Producer(string dest) : base(dest) { }
        public Producer(string dest, JMSDestType dType) : base(dest, dType) { }

        public override string configure(string brokerURL, string defaultUserName, string defaultPassword)
        {
            return configure(brokerURL, defaultUserName, defaultPassword, "", JMSDestType.TOPIC, "");
        }
        public override string configure(string brokerURL, string defaultUserName, string defaultPassword, string destNameIn, JMSDestType destTypeIn)
        {
            string status = base.configure(brokerURL, defaultUserName, defaultPassword, destNameIn, destTypeIn, "");
            if (status.StartsWith("ERROR"))
            {
                return status;
            }
            try
            {
                Producer = ProducerSession.createProducer(Destination);
            }
            catch (JMSException ex)
            {
                status = "ERROR Creating MessageProducer for " + Enum.GetName(DestType.GetType(), DestType) + " named '" + DestName + "'" + ex.ToString();
            }
            return status;
        }

        //public string Send(string data)
        //{
        //    try
        //    {
        //        //TODO: Add a more sofisticated message creation: i.e. setting message properties, etc 
        //        if (Producer != null)
        //        { Producer.send(ProducerSession.createTextMessage(data)); }
        //        else
        //        { return "ERROR sending: Poducer not initialized"; }

        //    }
        //    catch (JMSException ex)
        //    {
        //        return "ERROR sending '" + data + "' to '" + DestName + "' :" + ex.ToString();
        //    }
        //    return "Send successful";
        //}

        public string Send(string data)
        {
            return Send(data, JMSMessageType.TEXT, null, null);
        }

        public string Send(string data, JMSMessageType type)
        {
            return Send(data, type, null, null);
        }

        public string Send(object data, JMSMessageType type, Dictionary<JMSHeaderProps, Object> JMSProperties, Dictionary<string, Object> UserProperties)
        {
            try
            {
                Message msg = null;
                if (Producer != null) {
                    msg = makeMessage(type, data);
                    if (UserProperties != null)
                    {
                        foreach (string pkey in UserProperties.Keys)
                        {
                            object value = UserProperties[pkey];
                            msg.setObjectProperty(pkey, value);
                        }
                    }

                    if (JMSProperties!=null) {
                        foreach (JMSHeaderProps prop in JMSProperties.Keys) {
                            object value = JMSProperties[prop];
                            switch (prop) {
                                case JMSHeaderProps.JMSCorreleationID :
                                    msg.setJMSCorrelationID(value.ToString());
                                    break;
                                case JMSHeaderProps.JMSDeliveryMode :
                                    msg.setJMSDeliveryMode(Int32.Parse(value.ToString()));
                                    break;
                                case JMSHeaderProps.JMSDestination :
                                    msg.setJMSDestination((Destination)value);
                                    break;
                                case JMSHeaderProps.JMSExpiration :
                                    msg.setJMSExpiration(long.Parse(value.ToString()));
                                    break;
                                case JMSHeaderProps.JMSPriority :
                                    msg.setJMSPriority(Int32.Parse(value.ToString()));
                                    break;
                                case JMSHeaderProps.JMSReplyTo :
                                    msg.setJMSReplyTo((Destination)value);
                                    break;
                            }
                        }
                    }
                    Producer.send(msg); 
                }
                else
                { return "ERROR sending: Poducer not initialized"; }

            }
            catch (JMSException ex)
            {
                return "ERROR sending '" + data + "' to '" + DestName + "' :" + ex.ToString();
            }
            return "Send successful";
        }

        private Message makeMessage(JMSMessageType type, object data)
        {
            Message msg = null;
            switch (type)
            {
                case JMSMessageType.OBJECT:
                    msg = makeObjectMessage(data);
                    break;
                case JMSMessageType.BYTES:
                    msg = makeBytesMessage(data);
                    break;
                case JMSMessageType.STREAM:
                    msg = makeStreamMessage(data);
                    break;
                case JMSMessageType.MAP:
                    msg = makeMapMessage(data);
                    break;
                case JMSMessageType.MULTIPART:
                    msg = createMultipartMessage(data);
                    break;
                case JMSMessageType.TEXT:
                    msg = makeTextMessage(data);
                    break;
                case JMSMessageType.XML:
                    msg = makeXMLMessage(data);
                    break;
                default:
                    msg = makeTextMessage(data);
                    break;
            }
            return msg;
        }

        private Message createMultipartMessage(object data)
        {
            Message msg = ((Sonic.Jms.Ext.Session)ProducerSession).createMultipartMessage();
            if (data is List<object>)
            {
                foreach (object partData in ((List<object>)data))
                {
                    if (partData is Dictionary<string, PartData>)
                    {
                        createPartsFromDictionary(msg, (Dictionary<string, PartData>)partData);
                    }
                    else if (partData is PartData)
                    {
                        addOnePart(msg, (PartData)partData);
                    }
                }
            }
            else if (data is Dictionary<string, PartData>)
            {
                createPartsFromDictionary(msg, (Dictionary<string,PartData>)data);
            }
            else if (data is PartData)
            {
                addOnePart(msg, (PartData)data);
            }
            return msg;
        }

        private void createPartsFromDictionary(Message msg, Dictionary<string, PartData> partData)
        {
            foreach (string key in partData.Keys)
            {
                PartData pd = partData[key];
                addOnePart(msg, pd);
            }
        }

        private void addOnePart(Message msg, PartData pd)
        {
            Message onePart = makeMessage(pd.PartMsgType, pd.PartBody);
            if (pd.PartProps != null)
            {
                foreach (string pkey in pd.PartProps.Keys)
                {
                    object value = pd.PartProps[pkey];
                    onePart.setObjectProperty(pkey, value);
                }
            }
            Sonic.Jms.Ext.Part p = ((Sonic.Jms.Ext.MultipartMessage)msg).createMessagePart(onePart);
            p.getHeader().setContentId(pd.PartContentID);
            ((Sonic.Jms.Ext.MultipartMessage)msg).addPart(p);
        }

        private Message makeMapMessage(object data)
        {
            Message msg = ProducerSession.createMapMessage();
            if (data is Dictionary<string, object>)
            {
                foreach (string key in ((Dictionary<string, object>)data).Keys)
                {
                    object value = ((Dictionary<string, object>)data)[key];
                    ((MapMessage)msg).setObject(key, value);
                }
            }
            return msg;
        }

        private Message makeBytesMessage(object data)
        {
            Message msg = ProducerSession.createBytesMessage();
            if (data is byte[])
            {
                ((BytesMessage)msg).writeBytes((byte[])data);
            }
            else
            {
                System.IO.MemoryStream ms = new System.IO.MemoryStream();
                System.Runtime.Serialization.Formatters.Binary.BinaryFormatter bf1 =
                    new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
                bf1.Serialize(ms, data);
                ((BytesMessage)msg).writeBytes(ms.ToArray());
            }
            return msg;
        }
        private Message makeStreamMessage(object data)
        {
            Message msg = ProducerSession.createStreamMessage();
            if (data is byte[])
            {
                ((StreamMessage)msg).writeBytes((byte[])data);
            }
            else if (data is System.IO.MemoryStream)
            {
                ((StreamMessage)msg).writeBytes(((System.IO.MemoryStream)data).ToArray());
            }
            else
            {
                System.IO.MemoryStream ms = new System.IO.MemoryStream();
                System.Runtime.Serialization.Formatters.Binary.BinaryFormatter bf1 = 
                    new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
                bf1.Serialize(ms, data);
                ((StreamMessage)msg).writeBytes(ms.ToArray());
            }
            return msg;
        }

        private Message makeObjectMessage(object data)
        {
            Message msg = ProducerSession.createObjectMessage((System.Runtime.Serialization.ISerializable)data);
            return msg;
        }
        private Message makeTextMessage(object data)
        {
            Message msg = ProducerSession.createTextMessage(data.ToString());
            return msg;
        }
        private Message makeXMLMessage(object data)
        {
            Message msg = null;
            if (data is System.Xml.XmlDocument)
            {
                msg = ((Sonic.Jms.Ext.Session)ProducerSession).createXMLMessage((System.Xml.XmlDocument)data);
            }
            else
            {
                msg = ((Sonic.Jms.Ext.Session)ProducerSession).createXMLMessage(data.ToString());
            }
            return msg;
        }

    }
    [Serializable]
    public class PartData
    {

        public PartData(JMSMessageType partMsgTypeIn, Object partBodyIn)
        {
            partMsgType = partMsgTypeIn;
            partBody = partBodyIn;
        }
        
        public PartData(JMSMessageType partMsgTypeIn, Object partBodyIn,
            string partContentIn, Dictionary<string, object> partPropsIn)
        {
            partMsgType = partMsgTypeIn;
            partBody = partBodyIn;
            partContentID = partContentIn;
            partProps = partPropsIn;
        }

        string partContentID;

        public string PartContentID
        {
            get { return partContentID; }
            set { partContentID = value; }
        }

        JMSMessageType partMsgType;

        public JMSMessageType PartMsgType
        {
            get { return partMsgType; }
            set { partMsgType = value; }
        }

        Object partBody;

        public Object PartBody
        {
            get { return partBody; }
            set { partBody = value; }
        }

        Dictionary<string, object> partProps;

        public Dictionary<string, object> PartProps
        {
            get { return partProps; }
            set { partProps = value; }
        }

    }

}
