using System;
using System.Collections.Generic;
using System.Text;
using Sonic.Jms;
using System.Collections;
using System.Configuration;
//using global::sonicsw.pso.message.generator.Properties;

namespace sonicsw.pso.utilities
{
    public class JMSAccess : Sonic.Jms.MessageListener, Sonic.Jms.ExceptionListener, Sonic.Jms.Ext.ConnectionStateChangeListener
    {
        private static log4net.ILog log = log4net.LogManager.GetLogger(
            System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        #region Constructors
        public JMSAccess(IJMSAccessDisplay parentIn, System.Configuration.ApplicationSettingsBase defs)
        {
            parent = parentIn;
            defaultSettings = defs;
        }
        #endregion
       
        #region Properties
        IJMSAccessDisplay parent;

        public IJMSAccessDisplay Parent
        {
            get { return parent; }
            set { parent = value; }
        }

        private ApplicationSettingsBase defaultSettings;
        public ApplicationSettingsBase Default
        {
            get { return defaultSettings; }
            set { defaultSettings = value; }
        }

        public System.Collections.Hashtable CurrentProperties
        {
            get { return Parent.Props; }
        }

        Sonic.Jms.Ext.ConnectionFactory factory;
        public Sonic.Jms.Ext.ConnectionFactory Factory
        {
            get
            {
                if (factory == null)
                {
                    factory = setFactoryProperties();
                }
                return factory;
            }
            set
            {
                factory = setFactoryProperties();
                
            }
        }
        
        Connection connection;
        public Connection Connection
        {
            get
            {
                if (connection == null)
                {
                    connection = Factory.createConnection();
                    connection.setExceptionListener(this);
                    ((Sonic.Jms.Ext.Connection)connection)
                        .setConnectionStateChangeListener(this);
                }
                return connection;
            }
            set
            {
                if (connection != null)
                {
                    try
                    {
                       //connection.setExceptionListener(null);
                       // ((Sonic.Jms.Ext.Connection)connection)
                       //     .setConnectionStateChangeListener(null);
                       connection.close();
                    }
                    catch (Exception exception)
                    {
                        log.Debug("Error on Connection Close ",exception);
                    }
                }
                connection = value;
            }
        }
        
        Session producerSession;
        public Session ProducerSession
        {
            get
            {
                if (producerSession == null)
                {
                    //JMSAckMode currentAckMode = (JMSAckMode)CurrentProperties["AckMode"];
                    producerSession = Connection.createSession(false, 
                        (int)JMSAckMode.AUTO_ACKNOWLEDGE);
                }
                return producerSession;
            }
            set
            {
                if (producerSession != null)
                {
                    try
                    {
                        producerSession.close();
                    }
                    catch (Exception exception)
                    {
                        log.Debug("Error on ProducerSession Close ", exception);
                    }
                }
                producerSession = value;
            }
        }

        MessageProducer producer;
        public MessageProducer Producer
        {
            get
            {
                if (producer == null)
                {
                    producer = ProducerSession.createProducer(null);
                }
                return producer;
            }
            set
            {
                if (producer != null)
                {
                    try
                    {
                        producer.close();
                    }
                    catch (Exception exception)
                    {
                        log.Debug("Error on Producer Close ", exception);
                    }
                }
                producer = value;
            }
        }
        //bool destChanged = false;

        Session consumerSession;
        public Session ConsumerSession
        {
            get
            {
                if (consumerSession == null)
                {
                    JMSAckMode currentAckMode = (JMSAckMode)CurrentProperties["AckMode"];
                    //TODO: Add Transaction Support
                    consumerSession = Connection.createSession(false, (int)currentAckMode);
                }
                return consumerSession;
            }
            set
            {
                if (consumerSession != null)
                {
                    try
                    {
                        consumerSession.close();
                    }
                    catch (Exception exception)
                    {
                        log.Debug("Error on ConsumerSession Close ", exception);
                    }
                }
                consumerSession = value;
            }
        }

        MessageConsumer consumer;
        public MessageConsumer Consumer
        {
            get
            {
                if (consumer == null)
                {
                    //TODO: Add support for transactions durables and selectors 
                    consumer = ConsumerSession.createConsumer(ListenDest);
                    consumer.setMessageListener(this);
                    Connection.start();
                    ConsumerSession.recover();
                }
                return consumer;
            }
            set
            {
                if (consumer != null)
                {
                    try
                    {
                        Connection.stop();
                        consumer.setMessageListener(null);
                        consumer.close();
                    }
                    catch (Exception exception)
                    {
                        log.Debug("Error on Consumer Close ", exception);
                    }
                }
                consumer = value;
            }
        }
        //bool destChanged = false;



        public bool DestChanged
        {
            get { return (bool)CurrentProperties["DestChanged"]; }
            set {
                if (CurrentProperties.ContainsKey("DestChanged"))
                {
                    CurrentProperties["DestChanged"] = value;
                }
                else
                {
                    CurrentProperties.Add("DestChanged",value);
                }
            }
        }
        public bool ListenDestChanged
        {
            get { return (bool)CurrentProperties["ListenDestChanged"]; }
            set
            {
                if (CurrentProperties.ContainsKey("ListenDestChanged"))
                {
                    CurrentProperties["ListenDestChanged"] = value;
                }
                else
                {
                    CurrentProperties.Add("ListenDestChanged", value);
                }
            }
        }
       
        Destination dest;
        public Destination Dest
        {
            get
            {
                if (dest == null || DestChanged )
                {
                    bool queue = (bool)CurrentProperties["Queue"];
                    if (queue)
                    {
                        dest = ProducerSession.createQueue((string)CurrentProperties["SelectedDest"]);
                    }
                    else
                    {
                        dest = ProducerSession.createTopic((string)CurrentProperties["SelectedDest"]);
                    }
                    DestChanged = false;
                }
                return dest;
            }
            set { dest = value; }
        }

        Destination listenDest;
        public Destination ListenDest
        {
            get
            {
                if (listenDest == null || ListenDestChanged)
                {
                    bool queue = (bool)CurrentProperties["ListenQueue"];
                    if (queue)
                    {
                        listenDest = ConsumerSession.createQueue((string)CurrentProperties["SelectedListenDest"]);
                    }
                    else
                    {
                        listenDest = ConsumerSession.createTopic((string)CurrentProperties["SelectedListenDest"]);
                    }
                    ListenDestChanged = false;
                }
                return listenDest;
            }
            set { listenDest = value; }
        }

        #endregion

        #region Factory Population Methods
        
        public Sonic.Jms.Ext.ConnectionFactory setFactoryProperties()
        {
            System.Collections.Hashtable env = CurrentProperties;
            Sonic.Jms.Ext.ConnectionFactory factory = new Sonic.Jms.Cf.Impl.ConnectionFactory();
            factory.setClientID((string)env["ClientID"]);
            factory.setClientTransactionBufferSize(long.Parse(
                env["ClientTransactionBufferSize"].ToString()));
            factory.setConnectID((string)env["ConnectID"]);
            factory.setConnectionURLs((string)env["ConnectionURLs"]);
            factory.setDefaultPassword((string)env["DefaultPassword"]);
            factory.setDefaultTxnBatchSize(int.Parse(
                env["DefaultTxnBatchSize"].ToString()));
            factory.setDefaultUser((string)env["DefaultUser"]);
            factory.setDurableMessageOrder((bool)
                env["DurableMessageOrder"]);
            factory.setFaultTolerant((bool)env["FaultTolerant"]);
            factory.setFaultTolerantReconnectTimeout(int.Parse(
                env["FaultTolerantReconnectTimeout"].ToString()));
            FlowToDisk flow = (FlowToDisk)env["FlowToDisk"];
            factory.setFlowToDisk((int)flow);

            //factory.setHTTPCredentials(true,0,"object");
            //factory.setHTTPProxyInfo("host",0,true);
            factory.setInitialConnectTimeout(int.Parse(
                env["InitialConnectTimeout"].ToString()));
            factory.setLoadBalancing((bool)env["LoadBalancing"]);
            //factory.setLoginSPI(factory.getLoginSPI());
            //factory.setLoginSPILibrary(factory.getLoginSPI());
            factory.setMonitorInterval(int.Parse(
                env["MonitorInterval"].ToString()));
            factory.setPersistentDelivery((bool)env["PersistentDelivery"]);
            factory.setPingInterval(long.Parse(
                env["PingInterval"].ToString()));
            factory.setPrefetchCount(int.Parse(
                env["PrefetchCount"].ToString()));
            factory.setPrefetchThreshold(int.Parse(
                env["PrefetchThreshold"].ToString()));
            factory.setSelectorAtBroker((bool)env["SelectorAtBroker"]);
            factory.setSequential((bool)env["Sequential"]);
            //factory.setSSLCertificate(0, "certStoreName", 0, "certStoreInfo");
            //factory.setSSLCommonName();
            return factory;
        }


        public void loadFactoryDefaults()
        {
            loadFactoryDefaults(Factory);
        }


        public void loadFactoryDefaults(Sonic.Jms.Ext.ConnectionFactory factoryIn)
        {
            populateDefaults(Default);
            object prop = Utilities.TestValue("ClientID", Default);
            factoryIn.setClientID(prop==null?"":(string)prop);
            prop = Utilities.TestValue("ClientTransactionBufferSize", Default);
            factoryIn.setClientTransactionBufferSize(prop!=null ? long.Parse(""+prop) : -1L);

            prop = Utilities.TestValue("ConnectID", Default);
            factoryIn.setConnectID(prop == null ? "" : (string)prop);

            //factoryIn.setConnectionURLs(Default["ConnectionURLs"]);
            //factoryIn.setDefaultPassword(Default["DefaultPassword"]);
            factoryIn.setDefaultTxnBatchSize(int.Parse(""+
                Default["DefaultTxnBatchSize"]));
            //factoryIn.setDefaultUser(Default["DefaultUser"]);
            factoryIn.setDurableMessageOrder((bool)Default["DurableMessageOrder"]);
            factoryIn.setFaultTolerant((bool)Default["FaultTolerant"]);
            factoryIn.setFaultTolerantReconnectTimeout(int.Parse(""+
                Default["FaultTolerantReconnectTimeout"]));

            FlowToDisk flow = (FlowToDisk)
                Enum.Parse(typeof(FlowToDisk), (string)Default["FlowToDisk"], true);
            factoryIn.setFlowToDisk((int)flow);
            //factoryIn.setHTTPCredentials(true,0,"object");
            //factoryIn.setHTTPProxyInfo("host",0,true);
            factoryIn.setInitialConnectTimeout(int.Parse(""+Default["InitialConnectTimeout"]));
            factoryIn.setLoadBalancing((bool)Default["LoadBalancing"]);
            //factoryIn.setLoginSPI(factoryIn.getLoginSPI());
            //factoryIn.setLoginSPILibrary(factoryIn.getLoginSPI());
            factoryIn.setMonitorInterval(int.Parse(""+Default["MonitorInterval"]));
            factoryIn.setPersistentDelivery((bool)Default["PersistentDelivery"]);
            factoryIn.setPingInterval(long.Parse(""+Default["PingInterval"]));
            factoryIn.setPrefetchCount(int.Parse(""+Default["PrefetchCount"]));

            factoryIn.setPrefetchThreshold(int.Parse(""+Default["PrefetchThreshold"]));
            factoryIn.setSelectorAtBroker((bool)Default["SelectorAtBroker"]);
            factoryIn.setSequential((bool)Default["Sequential"]);
            //factoryIn.setSSLCertificate(0, "certStoreName", 0, "certStoreInfo");
            //factoryIn.setSSLCommonName();
        }



        ///  <summary>
        ///  populate defaults from class
        /// </summary>
        public static void populateDefaults(System.Configuration.ApplicationSettingsBase Default)
        {
            Default["Init"] = true;
            Sonic.Jms.Ext.ConnectionFactory factoryDef =
                new Sonic.Jms.Cf.Impl.ConnectionFactory();
            Default["ClientID"] =
                factoryDef.getClientID() == null ? "" : factoryDef.getClientID();
            Default["ClientTransactionBufferSize"] =
                   (decimal)factoryDef.getClientTransactionBufferSize();
            Default["ConnectID"] =
                factoryDef.getConnectID() == null ? "" : factoryDef.getConnectID();
            try
            {
                decimal dec = (decimal)factoryDef.getDefaultTxnBatchSize();
                Default["DefaultTxnBatchSize"] =
                       (decimal)factoryDef.getDefaultTxnBatchSize();
            }
            catch //(NullReferenceException ex)
            {
                Default["DefaultTxnBatchSize"] =
                       (decimal)-1;
            }
            Default["DurableMessageOrder"] =
                   factoryDef.getDurableMessageOrder();
            Default["FaultTolerant"] =
                   factoryDef.getFaultTolerant();
            Default["FaultTolerantReconnectTimeout"] =
                   (decimal)factoryDef.getFaultTolerantReconnectTimeout();
            Default["FlowToDisk"] =
                   Enum.GetName(typeof(FlowToDisk), factoryDef.getFlowToDisk());
            Default["InitialConnectTimeout"] =
                   (decimal)factoryDef.getInitialConnectTimeout();
            Default["LoadBalancing"] =
                   factoryDef.getLoadBalancing();
            Default["MonitorInterval"] =
                   (decimal)factoryDef.getMonitorInterval();
            Default["PersistentDelivery"] =
                    factoryDef.getPersistentDelivery();
            Default["PingInterval"] =
                   (decimal)factoryDef.getPingInterval();
            Default["PrefetchCount"] =
                   (decimal)factoryDef.getPrefetchCount();
            Default["PrefetchThreshold"] =
                   (decimal)factoryDef.getPrefetchThreshold();
            Default["SelectorAtBroker"] =
                    factoryDef.getSelectorAtBroker();
            Default["Sequential"] =
                    factoryDef.getSequential();
            Default["Init"] = false;
            Default.Save();
        }
        

        #endregion

        #region Process Messages
        
        public void connect()
        {
            MessageConsumer c = Consumer;
            MessageProducer p = Producer;
        }

        public void sendMessage(MessageRoot msgObj, StringBuilder msgBuffer)
        {
            Sonic.Jms.Message msg =
                ProducerSession.createTextMessage(msgBuffer.ToString());
            msg.setStringProperty("MessageKey", msgObj.Key);
            msg.setJMSCorrelationID(msgObj.Key.Replace(' ', '0'));
            msg.setJMSReplyTo(ProducerSession.createTopic("dev.Exit"));
            Producer.send(Dest, msg);
        }


        public static Dictionary<string, object> GetJMSHeaderProps(Message msg)
        {
            Dictionary<string, object> jmsHeaders = new Dictionary<string, object>();
            string[] names = Enum.GetNames(typeof(JMSHeaderProps));

            foreach (string name in names)
            {
                object value = Utilities.GetPropertyValue(msg, name);
                if (value != null) { jmsHeaders.Add(name, value); }
            }
            return jmsHeaders;

        }

        public static Dictionary<string, object> GetUserProps(Message message, Dictionary<string, object> headers)
        {
            string name = "";
            for (IEnumerator ie = message.getPropertyNames(); ie.MoveNext(); name = ie.Current.ToString())
            {
                headers.Add(name, message.getObjectProperty(name));
            }
           return headers;
        }

        //String name = Enum.GetName(typeof(JMSHeaderProps),
        //JMSHeaderProps.JMSCorrelationID);
        //object value = message.getJMSCorrelationID();
        //if (value != null) { jmsHeaders.Add(name, value); }

        //name = Enum.GetName(typeof(JMSHeaderProps),
        //        JMSHeaderProps.JMSDeliveryMode);
        //value = message.getJMSDeliveryMode();
        //if (value != null) { jmsHeaders.Add(name, value); }

        //name = Enum.GetName(typeof(JMSHeaderProps),
        //        JMSHeaderProps.JMSDestination);
        //value = message.getJMSDestination();
        //if (value != null) { jmsHeaders.Add(name, value); }

        //name = Enum.GetName(typeof(JMSHeaderProps),
        //        JMSHeaderProps.JMSExpiration);
        //value = message.getJMSExpiration();
        //if (value != null) { jmsHeaders.Add(name, value); }

        //name = Enum.GetName(typeof(JMSHeaderProps),
        //        JMSHeaderProps.JMSMessageID);
        //value = message.getJMSMessageID();
        //if (value != null) { jmsHeaders.Add(name, value); }

        //name = Enum.GetName(typeof(JMSHeaderProps),
        //        JMSHeaderProps.JMSPriority);
        //value = message.getJMSPriority();
        //if (value != null) { jmsHeaders.Add(name, value); }

        //name = Enum.GetName(typeof(JMSHeaderProps),
        //        JMSHeaderProps.JMSRedelivered);
        //value = message.getJMSRedelivered();
        //if (value != null) { jmsHeaders.Add(name, value); }

        //name = Enum.GetName(typeof(JMSHeaderProps),
        //        JMSHeaderProps.JMSReplyTo);
        //value = message.getJMSReplyTo();
        //if (value != null) { jmsHeaders.Add(name, value); }

        //name = Enum.GetName(typeof(JMSHeaderProps),
        //        JMSHeaderProps.JMSTimestamp);
        //value = message.getJMSTimestamp();
        //if (value != null) { jmsHeaders.Add(name, value); }

        //name = Enum.GetName(typeof(JMSHeaderProps),
        //        JMSHeaderProps.JMSType);
        //value = message.getJMSType();
        //if (value != null) { jmsHeaders.Add(name, value); }

        #endregion
        
        #region Clean Up
        public void cleanUp()
        {
            try
            {
                log.Debug("Clean Consumer ...");
                Consumer = null;
                log.Debug("Done");
                log.Debug("Clean ConsumerSession ...");
                ConsumerSession = null;
                log.Debug("Done");
                log.Debug("Clean Producer ...");
                Producer = null;
                log.Debug("Done");
                log.Debug("Clean ProducerSession ...");
                ProducerSession = null;
                log.Debug("Done");
                log.Debug("Clean Connection ...");
                Connection = null;
                log.Debug("Done");
                log.Debug("Clean Factory ...");
                Factory = null;
                log.Debug("Done");
            }
            catch (Exception ex)
            {
                log.Debug("Caught exception in cleanup ...",ex);
            }
        }
        #endregion
    
        #region MessageListener Members

        public void  onMessage(Message message)
        {
            //TODO: Add delegate notification of event
           log.InfoFormat("message : {0}", message.ToString());
           Parent.RaiseJMSMessage(this, new JMSMessageEventArgs(message));
        }

        #endregion

        #region ExceptionListener Members

        public void onException(JMSException exception)
        {
            //TODO: Add delegate notification of event
            Parent.RaiseJMSException(this, new JMSExceptionEventArgs(exception));
            log.Error("JMSException detected : " + exception.Message, exception);
        }

        #endregion

        #region ConnectionStateChangeListener Members

        public void connectionStateChanged(int state)
        {
            String stateStr = Enum.GetName(typeof(JmsConnectionStates), ((JmsConnectionStates)state));
            //TODO: Add delegate notification of event
            Parent.RaiseJMSConnectionState(this, 
                new JMSConnectionStateEventArgs(
                (JmsConnectionStates)state));
            log.InfoFormat("ConnectionStateChange detected : {0}", stateStr);
        }

        #endregion
    }
}
