/*------------------------------------------------------------------------
    File        : PubSub/TextPubSubReceive.p
    Purpose     : Receive a text message to a Sonic Topic 
  ----------------------------------------------------------------------*/

{common/connectToSonicBroker.i}

DEFINE VARIABLE hMsgConsumer AS HANDLE NO-UNDO.

/* Subscribe to SampleTopic1. Messages are handled by the
"messageHandler" internal procedure. */
RUN createMessageConsumer IN hSession (
  INPUT THIS-PROCEDURE,    /* this proc will handle the message  */
  INPUT "messageHandler",  /* name of internal procedure         */
  OUTPUT hMsgConsumer).    /* output the message consumer handle */
   
RUN subscribe IN hSession (
  INPUT "SampleTopic1", /* name of topic to subscribe to               */
  INPUT ?,              /* by default, the subscription is not durable */
  INPUT ?,              /* by defualt, no message selector             */
  INPUT FALSE,          /* receive copies of my own messages too       */
  INPUT hMsgConsumer).  /* handles the incoming messages               */

/* Start receiving messages */
RUN startReceiveMessages IN hSession.

DISPLAY "Waiting to receive Text message (SampleTopic1)...".

/* Wait to receive the messages. */
WAIT-FOR u1 OF THIS-PROCEDURE.

/* Clean up - Delete the session */
RUN deleteSession IN hSession.


PROCEDURE messageHandler:
  /* will be invoked when a message is received */
  DEFINE INPUT  PARAMETER phMessage     AS HANDLE.
  DEFINE INPUT  PARAMETER phMsgConsumer AS HANDLE.
  DEFINE OUTPUT PARAMETER phReply       AS HANDLE.
  
  DEFINE VARIABLE lcMessageText AS LONGCHAR NO-UNDO
    VIEW-AS EDITOR LARGE SIZE 70 BY 5.

  /* get the body of the text message */  
  lcMessageText = DYNAMIC-FUNCTION('getText':U IN phMessage).

  /* process the message */
  DISPLAY "Message text: " lcMessageText
    WITH NO-LABELS.

  /* delete the message */
  RUN deleteMessage IN phMessage.
  
  /* close this procedure */
  APPLY "U1" TO THIS-PROCEDURE.
END.

